# -*- coding: utf-8 -*-
import logging

from .basicaction import BasicAction


class LogAction(BasicAction):
    """Action to log a paste to console"""
    name = "LogAction"

    def __init__(self):
        super().__init__()
        self.logger = logging.getLogger(__name__)

    def perform(self, paste, analyzer_name=None):
        self.logger.debug("New Paste matched: {0}".format(paste))



# -*- coding: utf-8 -*-
import logging
import os
import sqlite3

from .abstractdb import AbstractDB


class SQLiteDB(AbstractDB):

    def __init__(self, dbpath="pastepwn"):
        super().__init__()
        self.dbpath = dbpath
        self.logger = logging.getLogger(__name__)
        self.logger.debug("Initializing SQLite - {0}".format(dbpath))

        # Check if the folder path exists
        if not os.path.exists(os.path.dirname(dbpath)):
            # If not, create the path and the file
            os.mkdir(os.path.dirname(dbpath))
            open(self.dbpath, "a").close()

        try:
            self.db = sqlite3.connect(dbpath)
            self.db.text_factory = lambda x: str(x, 'utf-8', "ignore")
            self.cursor = self.db.cursor()
            self._create_tables()
        except Exception as e:
            self.logger.exception("An exception happened when initializing the database: {0}".format(e))
            raise

        self.logger.debug("Connected to database!")

    def _create_tables(self):
        open(self.dbpath, "a").close()

        self.cursor.execute("""CREATE TABLE IF NOT EXISTS 'pastes' (
                            'key'	TEXT NOT NULL UNIQUE,
                            'title'	TEXT,
                            'user'	TEXT,
                            'size'	INTEGER,
                            'date'	INTEGER,
                            'expire'	INTEGER,
                            'scrape_url'	TEXT,
                            'full_url'	TEXT,
                            'body'	TEXT,
                            PRIMARY KEY('key'))""")
        self.db.commit()

    def _insert_data(self, paste):
        self.cursor.execute("INSERT INTO pastes (key, title, user, size, date, expire, syntax, scrape_url, full_url, body) "
                            "VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)",
                            (paste.key,
                             paste.title,
                             paste.user,
                             paste.size,
                             paste.date,
                             paste.expire,
                             paste.syntax,
                             paste.scrape_url,
                             paste.full_url,
                             paste.body))
        self.db.commit()

    def _get_data(self, key, value):
        pass

    def count(self, key, value):
        # TODO add filter to counting
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def count_all(self):
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def store(self, paste):
        self.logger.debug("Storing paste {0}".format(paste.key))

        try:
            self._insert_data(paste)
        except Exception as e:
            self.logger.debug("Exception '{0}'".format(e))

    def get(self, key):
        return self._get_data("key", key)



# -*- coding: utf-8 -*-
import re

from .regexanalyzer import RegexAnalyzer


class IBANAnalyzer(RegexAnalyzer):
    """Analyzer to match International Bank Account Numbers (IBAN)"""
    name = "IBANAnalyzer"

    def __init__(self, actions, validate=False):
        # Regex adapted from https://stackoverflow.com/a/44657292/3621482
        regex = r"([A-Z]{2}[ \-]?[0-9]{2})(?!=(?:[ \-]?[A-Z0-9]){9,30}$)" \
                r"((?:[ \-]?[A-Z0-9]{3,5}){2,7})([ \-]?[A-Z0-9]{1,3})?"
        super().__init__(actions, regex)
        self.validate = validate

    def match(self, paste):
        paste_content = paste.body
        search_result = self.regex.search(paste_content)

        if search_result is None:
            return False

        if self.validate:
            matched_string = search_result.group()
            return self._validate_iban(matched_string)

        return True

    def _validate_iban(self, potential_iban):
        """Checks if the given string could be a valid IBAN. Adapted from https://rosettacode.org/wiki/IBAN#Python."""

        # Ensure upper alphanumeric input
        potential_iban = potential_iban.replace(' ', '').replace('\t', '').replace('-', '')
        if not re.match(r"^[\dA-Z]+$", potential_iban):
            return False

        # Validate if length matches the expected length based on the country code
        country_code = potential_iban[:2]
        if len(potential_iban) != self._iban_length_by_country.get(country_code, None):
            return False

        # Official validation ( https://en.wikipedia.org/wiki/International_Bank_Account_Number#Validating_the_IBAN )
        rearranged_iban = potential_iban[4:] + potential_iban[:4]
        integer_iban = int(''.join(str(int(ch, 36)) for ch in rearranged_iban))  # BASE 36: 0..9,A..Z -> 0..35
        return integer_iban % 97 == 1

    _iban_length_by_country = dict(
        AL=28, AD=24, AT=20, AZ=28, BE=16, BH=22, BA=20, BR=29,
        BG=22, CR=21, HR=21, CY=28, CZ=24, DK=18, DO=28, EE=20,
        FO=18, FI=18, FR=27, GE=22, DE=22, GI=23, GR=27, GL=18,
        GT=28, HU=28, IS=26, IE=22, IL=23, IT=27, KZ=20, KW=30,
        LV=21, LB=28, LI=21, LT=20, LU=20, MK=19, MT=31, MR=27,
        MU=30, MC=27, MD=24, ME=22, NL=18, NO=15, PK=24, PS=29,
        PL=28, PT=25, RO=24, SM=27, SA=24, RS=22, SK=24, SI=19,
        ES=24, SE=24, CH=21, TN=24, TR=26, AE=23, GB=22, VG=24)



# -*- coding: utf-8 -*-


class AbstractDB(object):

    def __init__(self):
        pass

    def store(self, paste):
        """Stores a paste in the database"""
        raise NotImplementedError

    def get(self, key):
        """Fetches a paste from the database"""
        raise NotImplementedError



# -*- coding: utf-8 -*-
from threading import Thread, current_thread
import logging


def start_thread(target, name, exception_event, *args, **kwargs):
    thread = Thread(target=thread_wrapper, name=name, args=(target, exception_event) + args, kwargs=kwargs)
    thread.start()
    return thread


def thread_wrapper(target, exception_event, *args, **kwargs):
    thread_name = current_thread().name
    logger = logging.getLogger(__name__)
    logger.debug('{0} - thread started'.format(thread_name))
    try:
        target(*args, **kwargs)
    except Exception:
        exception_event.set()
        logger.exception('unhandled exception in %s', thread_name)
        raise
    logger.debug('{0} - thread ended'.format(thread_name))


def join_threads(threads):
    """End all threads and join them back into the main thread"""
    logger = logging.getLogger(__name__)
    for thread in threads:
        logger.debug("Joining thread {0}".format(thread.name))
        thread.join()
        logger.debug("Thread {0} has ended".format(thread.name))



# -*- coding: utf-8 -*-

from .basicaction import BasicAction
from .savefileaction import SaveFileAction
from .telegramaction import TelegramAction
from .logaction import LogAction
from .genericaction import GenericAction
from .databaseaction import DatabaseAction
from .savejsonaction import SaveJSONAction

__all__ = ('BasicAction', 'SaveFileAction', 'TelegramAction', 'LogAction', 'GenericAction', 'DatabaseAction', 'SaveJSONAction')



# -*- coding: utf-8 -*-

from .request import Request
from .dictwrapper import DictWrapper
from .threadingutils import start_thread, join_threads

__all__ = ('Request', 'DictWrapper', 'start_thread', 'join_threads')



# -*- coding: utf-8 -*-
from .alwaystrueanalyzer import AlwaysTrueAnalyzer
from .basicanalyzer import BasicAnalyzer
from .bcrypthashanalyzer import BcryptHashAnalyzer
from .md5hashanalyzer import MD5HashAnalyzer
from .shahashanalyzer import SHAHashAnalyzer
from .creditcardanalyzer import CreditCardAnalyzer
from .genericanalyzer import GenericAnalyzer
from .mailanalyzer import MailAnalyzer
from .pastebinurlanalyzer import PastebinURLAnalyzer
from .regexanalyzer import RegexAnalyzer
from .urlanalyzer import URLAnalyzer
from .wordanalyzer import WordAnalyzer
from .ibananalyzer import IBANAnalyzer

__all__ = (
    'AlwaysTrueAnalyzer',
    'BasicAnalyzer',
    'BcryptHashAnalyzer',
    'MD5HashAnalyzer',
    'SHAHashAnalyzer',
    'CreditCardAnalyzer',
    'GenericAnalyzer',
    'MailAnalyzer',
    'PastebinURLAnalyzer',
    'RegexAnalyzer',
    'URLAnalyzer',
    'WordAnalyzer',
    'IBANAnalyzer'
)



# -*- coding: utf-8 -*-
from sanic import Sanic
from sanic.response import json


async def default(request):
    return json({"hello": "world"})


async def ignore_404s(request, exception):
    return json({"error": "site not found"}, status=404)


async def get_paste_by_id(request, pasteId):
    return json({"pasteId": pasteId})


async def get_pastes_by_date(request):
    return json({"hello1": "world2"})


async def scrape_paste(request):
    return json({"hello1": "world2"})


async def pastepwn(request):
    return json({"hello1": "world2"})



# -*- coding: utf-8 -*-

import logging
from queue import Empty, Queue
from threading import Event, Lock
from time import sleep

from pastepwn.util import start_thread, join_threads


class ActionHandler(object):
    """Handler to execute all the actions, if an analyzer matches a paste"""

    def __init__(self, action_queue=None, exception_event=None, stop_event=None):
        self.logger = logging.getLogger(__name__)
        self.action_queue = action_queue or Queue()
        self.__exception_event = exception_event or Event()
        self.__stop_event = stop_event or Event()
        self.__threads = []

        self.running = False
        self.__lock = Lock()

    def start(self, ready=None):
        """
        Starts the actionhandler to execute actions if pastes are matched
        :param ready: Event to check from the outside if the actionhandler has been started
        :return: None
        """
        with self.__lock:
            if not self.running:
                self.running = True

                thread = start_thread(self._start, "ActionHandler", self.__exception_event)
                self.__threads.append(thread)

            if ready is not None:
                ready.set()

    def stop(self):
        """
        Stops the actionhandler
        :return: None
        """
        self.__stop_event.set()
        while self.running:
            sleep(0.1)
        self.__stop_event.clear()

        join_threads(self.__threads)
        self.__threads = []

    def _start(self):
        while self.running:
            try:
                # Get paste from queue
                actions, paste, analyzer = self.action_queue.get(True, 1)
                if actions is None:
                    continue

                for action in actions:
                    self.logger.debug("Performing action '{0}' on paste '{1}' matched by analyzer '{2}'!".format(action.name, paste.key, analyzer.identifier))
                    action.perform(paste, analyzer.identifier)
            except Empty:
                if self.__stop_event.is_set():
                    self.logger.debug("orderly stopping ActionHandler")
                    self.running = False
                    break
                elif self.__exception_event.is_set():
                    self.logger.critical("stopping ActionHandler due to exception in another thread")
                    self.running = False
                    break



# -*- coding: utf-8 -*-
import re


class IPNotRegisteredError(Exception):

    def __init__(self, body):
        ip = re.search("YOUR IP: (.*?) DOES NOT HAVE ACCESS", body).group(1)
        super().__init__("The IP you use for scraping ({0}) was not whitelisted. Visit https://pastebin.com/doc_scraping_api to get access!".format(ip))



# -*- coding: utf-8 -*-

from .actionhandler import ActionHandler
from .scrapinghandler import ScrapingHandler
from .pastedispatcher import PasteDispatcher
from .pastepwn import PastePwn
from .paste import Paste

__all__ = ('PastePwn', 'ActionHandler', 'ScrapingHandler', 'PasteDispatcher', 'Paste')



# -*- coding: utf-8 -*-
from pastepwn.util import Request
from .basicaction import BasicAction


class WebhookAction(BasicAction):
    """Action to perform a Webhook on a matched paste"""
    name = "WebhookAction"

    def __init__(self, url, post_data=None):
        """
        Init method for the WebhookAction
        :param url: string, URL to POST against
        :param post_data: boolean, to decide wheather a paste should be sent in the body
        """
        super().__init__()
        self.url = url
        self.post_data = post_data

    def perform(self, paste, analyzer_name=None):
        """
        Trigger the webhook
        :param paste: The paste passed by the ActionHandler
        :param analyzer_name: The name of the analyzer which matched the paste
        :return: None
        """
        if self.post_data is None:
            paste_dict = None
        else:
            paste_dict = paste.to_dict()

        r = Request()
        r.post(url=self.url, data=paste_dict)



# -*- coding: utf-8 -*-


class Paste(object):

    def __init__(self, key, title, user, size, date, expire, syntax, scrape_url, full_url):
        self.key = key
        self.title = title
        self.user = user
        self.size = size
        self.date = date
        self.expire = expire
        self.syntax = syntax
        self.scrape_url = scrape_url
        self.full_url = full_url
        self.body = None

    def set_body(self, body):
        if body is None:
            body = ""
        self.body = str(body)

    def __str__(self):
        paste_dict = self.to_dict()
        return str(paste_dict)

    def __repr__(self):
        return self.__str__()

    def to_dict(self):
        paste_dict = {'scrape_url': self.scrape_url,
                      'full_url': self.full_url,
                      'date': self.date,
                      'key': self.key,
                      'size': self.size,
                      'expire': self.expire,
                      'title': self.title,
                      'syntax': self.syntax,
                      'user': self.user,
                      'body': self.body}
        return paste_dict



# -*- coding: utf-8 -*-
import logging

from .basicanalyzer import BasicAnalyzer


class GenericAnalyzer(BasicAnalyzer):
    """Analyzer to pass a function pointer to, in order to create an analyzer on the fly"""
    name = "GenericAnalyzer"

    def __init__(self, actions, match_func):
        super().__init__(actions)

        if match_func is None:
            raise ValueError("Function to be called cannot be None")

        self.match_func = match_func

    def match(self, paste):
        """Run the passed function and return its return value"""
        try:
            result = self.match_func(paste)
        except Exception as e:
            result = False
            logging.getLogger(__name__).warning("Executing custom match function raised an exception! {}".format(e))

        return result



# -*- coding: utf-8 -*-
from .urlanalyzer import URLAnalyzer


class PastebinURLAnalyzer(URLAnalyzer):
    """ Analyzer to match Pastebin URLs via regex """
    name = "PastebinURLAnalyzer"

    def __init__(self, actions, resolve=False):
        regex = r"((?:https?:\/\/)?pastebin.com\/\S+)"
        super().__init__(actions, regex, resolve)



# -*- coding: utf-8 -*-


class BasicAction(object):
    """Base class for actions which can be performed on pastes"""
    name = "BasicAction"

    def __init__(self):
        pass

    def perform(self, paste, analyzer_name=None):
        """Perform the action on the passed paste"""
        raise NotImplementedError



# -*- coding: utf-8 -*-


# https://stackoverflow.com/questions/19799609/leaving-values-blank-if-not-passed-in-str-format
class DictWrapper(dict):
    def __missing__(self, key):
        return '{' + key + '}'



# -*- coding: utf-8 -*-
import asyncio
import logging
import socket
from threading import Event, Lock

from sanic import Sanic
from sanic.exceptions import NotFound

from pastepwn.util import start_thread
from .apiroutes import get_paste_by_id, get_pastes_by_date, scrape_paste, default, pastepwn, ignore_404s


# on linux we can use
# import uvloop


class APIServer(object):

    def __init__(self, host="0.0.0.0", port=8080, base_url="", exception_event=None):
        self.logger = logging.getLogger(__name__)
        self.host = host
        self.port = port
        self.is_running = False

        if base_url.endswith("/"):
            base_url = base_url[:-1]

        self.base_url = base_url
        self._server_lock = Lock()
        self._exception_event = exception_event or Event()
        self.loop = None
        self._api_thread = None
        self.app = Sanic("APIServer")

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.bind((host, port))

        self._init_routes()

    def _init_routes(self):
        self._init_route(default, "/")
        self._init_route(get_paste_by_id, "/pastes/<pasteId>")
        self._init_route(get_pastes_by_date, "/pastes/findByDate")
        self._init_route(scrape_paste, "/pastes/scrape", methods=["POST"])
        self._init_route(pastepwn, "/pastepwn")
        self.app.error_handler.add(NotFound, ignore_404s)

    def _init_route(self, callback, path, methods=None):
        allowed_methods = methods or ["GET"]

        if path.startswith("/"):
            path = path[1:]

        self.app.add_route(callback, "{0}/{1}".format(self.base_url, path, methods=allowed_methods))

    def start(self):
        with self._server_lock:
            if self.is_running:
                return

            self._api_thread = start_thread(self._start_server, "sanicAPI", Event())
            self.is_running = True

    def stop(self):
        with self._server_lock:
            if not self.is_running:
                return

            try:
                self.loop.stop()
                self.logger.info("Stopping API Server!")
                self._api_thread.join()
                self.loop = None
                self.is_running = False
            except Exception as e:
                self.logger.error(e)

    def _start_server(self):
        asyncio.set_event_loop(asyncio.new_event_loop())
        server = self.app.create_server(sock=self.sock)
        self.loop = asyncio.get_event_loop()
        asyncio.ensure_future(server)

        try:
            self.loop.run_forever()
        except Exception as e:
            print(e)
            self.loop.stop()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class MD5HashAnalyzer(RegexAnalyzer):
    """Analyzer to match MD5 password hashes via regex"""
    name = "MD5HashAnalyzer"

    def __init__(self, actions):
        regex = r"[a-f0-9]{32}"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-

import logging
from threading import Event, Lock

from tornado.httpserver import HTTPServer
from tornado.ioloop import IOLoop
from tornado.web import RequestHandler, Application

from pastepwn.util import start_thread

logger = logging.getLogger(__name__)


class APIServer(object):

    def __init__(self, port, app, ssl_ctx, exception_event=None):
        self.http_server = HTTPServer(app, ssl_options=ssl_ctx)
        self.port = port
        self.loop = None
        self.logger = logging.getLogger(__name__)
        self.is_running = False
        self.server_lock = Lock()
        self.shutdown_lock = Lock()
        self._exception_event = exception_event or Event()
        self._api_thread = None

    def start(self):
        if self.is_running:
            logger.warning("APIServer is already running!")
            return

        self._api_thread = start_thread(self._start, "pastepwnAPI", Event())

    def _start(self):
        with self.server_lock:
            IOLoop().make_current()
            self.is_running = True
            self.logger.debug('API Server started.')
            self.http_server.listen(self.port)
            self.loop = IOLoop.current()
            self.loop.start()
            self.logger.debug('API Server stopped.')
            self.is_running = False

    def stop(self):
        with self.shutdown_lock:
            if not self.is_running:
                self.logger.warning('API Server already stopped.')
                return
            else:
                # self.http_server.close_all_connections()
                # TODO Somehow the sockets keep existing which is why at some point there will
                # TODO be an OS error because the library tries to open another socked which already exists
                self.http_server.stop()
                self.loop.add_callback(self.loop.stop)

    def handle_error(self, request, client_address):
        """Handle an error gracefully."""
        self.logger.debug('Exception happened during processing of request from %s',
                          client_address, exc_info=True)


class APIAppClass(Application):

    def __init__(self, base_path="api"):
        handlers = [
            (r"/{0}/pastes/(.*)".format(base_path), PasteHandler)
        ]
        Application.__init__(self, handlers)


class PasteHandler(RequestHandler):
    SUPPORTED_METHODS = ["GET"]

    def __init__(self, application, request, **kwargs):
        super(PasteHandler, self).__init__(application, request, **kwargs)
        self.logger = logging.getLogger(__name__)

    def set_default_headers(self):
        self.set_header("Content-Type", 'application/json; charset="utf-8"')

    def get(self, *args, **kwargs):
        # TODO Remove debug code
        print(self.path_args)
        self.set_status(200)
        logger.info(self.request.body)
        self.write("Hi")



# -*- coding: utf-8 -*-
import logging
from queue import Queue
from threading import Event

from pastepwn.actions import DatabaseAction
from pastepwn.analyzers import AlwaysTrueAnalyzer
from pastepwn.core import ScrapingHandler, ActionHandler
from pastepwn.core.pastedispatcher import PasteDispatcher
from pastepwn.util.request import Request


class PastePwn(object):

    def __init__(self, database=None, proxies=None):
        self.logger = logging.getLogger(__name__)
        self.database = database
        self.paste_queue = Queue()
        self.action_queue = Queue()
        self.__exception_event = Event()
        self.__request = Request(proxies)  # initialize singleton

        # Usage of ipify to get the IP - Uses the X-Forwarded-For Header which might
        # lead to issues with proxies
        try:
            ip = self.__request.get("https://api.ipify.org")
            self.logger.info("Your public IP is {0}".format(ip))
        except Exception as e:
            self.logger.warning("Could not fetch public IP via ipify: {0}".format(e))

        self.scraping_handler = ScrapingHandler(paste_queue=self.paste_queue,
                                                exception_event=self.__exception_event)
        self.paste_dispatcher = PasteDispatcher(paste_queue=self.paste_queue,
                                                action_queue=self.action_queue,
                                                exception_event=self.__exception_event)
        self.action_handler = ActionHandler(action_queue=self.action_queue,
                                            exception_event=self.__exception_event)

        if self.database is not None:
            # Save every paste to the database with the AlwaysTrueAnalyzer
            self.logger.info("Database provided! Storing pastes in there.")
            database_action = DatabaseAction(self.database)
            always_true = AlwaysTrueAnalyzer(database_action)
            self.add_analyzer(always_true)
        else:
            self.logger.info("No database provided!")

    def add_scraper(self, scraper):
        scraper.init_exception_event(self.__exception_event)
        self.scraping_handler.add_scraper(scraper)

    def add_analyzer(self, analyzer):
        self.paste_dispatcher.add_analyzer(analyzer)

    def start(self):
        if self.__exception_event.is_set():
            self.logger.error("An exception occured. Aborting the start of PastePwn!")
            exit(1)

        self.scraping_handler.start()
        self.paste_dispatcher.start()
        self.action_handler.start()

    def stop(self):
        self.scraping_handler.stop()
        self.paste_dispatcher.stop()
        self.action_handler.stop()




# -*- coding: utf-8 -*-

from .abstractdb import AbstractDB
from .mongodb import MongoDB
from .sqlitedb import SQLiteDB
from .mysqldb import MysqlDB

__all__ = ('AbstractDB', 'MongoDB', 'SQLiteDB', 'MysqlDB')



# -*- coding: utf-8 -*-

import logging
from queue import Empty, Queue
from threading import Event, Lock
from time import sleep

from pastepwn.util import start_thread, join_threads


class PasteDispatcher(object):

    def __init__(self, paste_queue, action_queue=None, exception_event=None):
        self.logger = logging.getLogger(__name__)
        self.paste_queue = paste_queue
        self.action_queue = action_queue or Queue()
        self.analyzers = []
        self.running = False

        self.__lock = Lock()
        self.__threads = []
        self.__thread_pool = set()
        self.__exception_event = exception_event or Event()
        self.__stop_event = Event()

    def _pool_thread(self):
        while True:
            pass

    def add_analyzer(self, analyzer):
        self.analyzers.append(analyzer)

    def start(self, workers=4, ready=None):
        """Starts dispatching the downloaded pastes to the list of analyzers"""
        with self.__lock:
            if not self.running:
                if len(self.analyzers) == 0:
                    self.logger.warning("No analyzers added! At least one analyzer must be added prior to use!")
                    return None

                self.running = True
                thread = start_thread(self._start_analyzing, "PasteDispatcher", exception_event=self.__exception_event)
                self.__threads.append(thread)

                # Start thread pool with worker threads
                # for i in range(workers):
                #    thread = Thread(target=self._pool_thread, name="analyzer_{0}".format(i))
                #    self.__thread_pool.add(thread)
                #    thread.start()

            if ready is not None:
                ready.set()

            return self.action_queue

    def _start_analyzing(self):
        while self.running:
            try:
                # Get paste from queue
                paste = self.paste_queue.get(True, 1)

                # TODO implement thread pool to limit number of parallel executed threads
                # Don't add these threads to the list. Otherwise they will just block the list
                start_thread(self._process_paste, "process_paste", paste=paste, exception_event=self.__exception_event)
            except Empty:
                if self.__stop_event.is_set():
                    self.logger.debug("orderly stopping")
                    self.running = False
                    break
                elif self.__exception_event.is_set():
                    self.logger.critical("stopping due to exception in another thread")
                    self.running = False
                    break
                continue

    def _process_paste(self, paste):
        self.logger.debug("Analyzing Paste: {0}".format(paste.key))
        for analyzer in self.analyzers:
            if analyzer.match(paste):
                actions = analyzer.actions
                self.action_queue.put((actions, paste, analyzer))

    def stop(self):
        self.__stop_event.set()
        while self.running:
            sleep(0.1)
        self.__stop_event.clear()

        join_threads(self.__threads)
        self.__threads = []



# -*- coding: utf-8 -*-
import re

from .basicanalyzer import BasicAnalyzer


class RegexAnalyzer(BasicAnalyzer):
    """Analyzer to match the content of a paste via regular expressions"""
    name = "RegexAnalyzer"

    def __init__(self, actions, regex):
        super().__init__(actions)
        self.regex = re.compile(regex)

    def match(self, paste):
        """Match the content of a paste via regex. Return true if regex matches"""
        paste_content = paste.body or ""
        return self.regex.search(paste_content) is not None



# -*- coding: utf-8 -*-
import json
import logging
import time
from queue import Queue, Empty

from pastepwn.core import Paste
from pastepwn.scraping import BasicScraper
from pastepwn.scraping.pastebin.exceptions import IPNotRegisteredError, EmptyBodyException
from pastepwn.util import Request, start_thread


# https://pastebin.com/doc_scraping_api#2
# Your whitelisted IP should not run into any issues as long as you don't abuse our service.
# We recommend not making more than 1 request per second, as there really is no need to do so.
# Going over 1 request per second won't get you blocked, but if we see excessive unnecessary scraping, we might take action.


class PastebinScraper(BasicScraper):
    name = "PastebinScraper"
    api_base_url = "https://scrape.pastebin.com"

    def __init__(self, paste_queue=None, exception_event=None, api_hit_rate=None):
        super().__init__(exception_event)
        self.logger = logging.getLogger(__name__)
        self._last_scrape_time = 0
        self.paste_queue = paste_queue or Queue()
        self._tmp_paste_queue = Queue()

        self._known_pastes = []
        self._known_pastes_limit = 1000

        # The hit rate describes the interval between two requests in seconds
        self._api_hit_rate = api_hit_rate or 1

    def _check_error(self, body):
        """Checks if an error occurred and raises an exception if it did"""
        if body is None:
            raise EmptyBodyException()

        if "DOES NOT HAVE ACCESS" in body:
            self._exception_event.set()
            raise IPNotRegisteredError(body)

    def _get_recent(self, limit=100):
        """Downloads a list of the most recent pastes - the amount is limited by the <limit> parameter"""
        r = Request()
        endpoint = "api_scraping.php"
        api_url = "{0}/{1}?limit={2}".format(self.api_base_url, endpoint, limit)

        try:
            response_data = r.get(api_url)

            self._check_error(response_data)

            pastes_dict = json.loads(response_data)
            pastes = []

            # Loop through the response and create objects by the data
            for paste in pastes_dict:
                paste_obj = Paste(key=paste.get("key"),
                                  title=paste.get("title"),
                                  user=paste.get("user"),
                                  size=paste.get("size"),
                                  date=paste.get("date"),
                                  expire=paste.get("expire"),
                                  syntax=paste.get("syntax"),
                                  scrape_url=paste.get("scrape_url"),
                                  full_url=paste.get("full_url"))
                pastes.append(paste_obj)

            return pastes
        except Exception as e:
            self.logger.error(e)
            return None

    def _get_paste_content(self, key):
        """Downloads the content of a certain paste"""
        r = Request()
        endpoint = "api_scrape_item.php"
        api_url = "{0}/{1}?i={2}".format(self.api_base_url, endpoint, key)
        content = ""

        self.logger.debug("Downloading paste {0}".format(key))
        try:
            response_data = r.get(api_url)

            self._check_error(response_data)

            content = response_data
        except Exception as e:
            self.logger.error(e)

        return content

    def _body_downloader(self):
        """Downloads the body of pastes from pastebin, which have been put into the queue"""
        while self.running:
            try:
                if self._tmp_paste_queue.qsize() > 0:
                    self.logger.debug("Queue size: {}".format(self._tmp_paste_queue.qsize()))

                if self._stop_event.is_set() or self._exception_event.is_set():
                    self.running = False
                    break

                paste = self._tmp_paste_queue.get(True, 1)

                # if paste is not known, download the body and put it on the queue and into the list
                last_body_download_time = round(time.time(), 2)
                body = self._get_paste_content(paste.key)

                paste.set_body(body)
                self.paste_queue.put(paste)

                current_time = round(time.time(), 2)
                diff = round(current_time - last_body_download_time, 2)

                if diff >= self._api_hit_rate:
                    continue

                sleep_diff = round(self._api_hit_rate - diff, 3)
                self.logger.debug("Sleep time is: {0}".format(sleep_diff))
                time.sleep(sleep_diff)
            except Empty:
                continue

    def start(self, paste_queue):
        """Start the scraping process and download the paste metadata"""
        self.paste_queue = paste_queue
        self.running = True
        start_thread(self._body_downloader, "BodyDownloader", self._exception_event)

        while self.running:
            self._last_scrape_time = int(time.time())
            pastes = self._get_recent(limit=100)
            counter = 0

            if pastes is not None:
                for paste in pastes:
                    # check if paste is in list of known pastes
                    if paste.key in self._known_pastes:
                        # Do nothing, if it's already known
                        continue

                    self.logger.debug("Paste is unknown - adding ot to list {}".format(paste.key))
                    self._tmp_paste_queue.put(paste)
                    self._known_pastes.append(paste.key)
                    counter += 1

                    if self._stop_event.is_set() or self._exception_event.is_set():
                        self.running = False
                        break

                self.logger.debug("{0} new pastes fetched!".format(counter))

            # Delete some of the last pastes to not run into memory/performance issues
            if len(self._known_pastes) > 1000:
                self.logger.debug("known_pastes > 1000 - cleaning up!")
                start_index = len(self._known_pastes) - self._known_pastes_limit
                self._known_pastes = self._known_pastes[start_index:]

            if self._stop_event.is_set() or self._exception_event.is_set():
                self.logger.debug('stopping {0}'.format(self.name))
                self.running = False
                break

            # check if time since last
            current_time = int(time.time())
            diff = current_time - self._last_scrape_time

            # if the last scraping happened less than 60 seconds ago,
            # wait until the 60 seconds passed
            if diff < 60:
                sleep_time = 60 - diff
                time.sleep(sleep_time)



# -*- coding: utf-8 -*-
import json
import os

from .savefileaction import SaveFileAction


class SaveJSONAction(SaveFileAction):
    """Action to save a json formatted file to the disk"""
    name = "SaveJSONAction"

    def __init__(self, path):
        super().__init__(path)

    def perform(self, paste, analyzer_name=None):
        if not os.path.exists(self.path):
            os.makedirs(self.path)

        with open(os.path.join(self.path, "{0}.json".format(paste.key)), "w") as file:
            file.write(json.dumps(paste.to_dict()))



from .core.pastepwn import PastePwn
from .core.paste import Paste
from .core.pastedispatcher import PasteDispatcher
from .core.scrapinghandler import ScrapingHandler
from .core.actionhandler import ActionHandler

__author__ = "d-Rickyy-b (pastepwn@rickyy.de)"

__all__ = ['PastePwn', 'Paste', 'PasteDispatcher', 'ScrapingHandler', 'ActionHandler']



# -*- coding: utf-8 -*-
from .basicaction import BasicAction


class DatabaseAction(BasicAction):
    """Action to save a paste to a database"""
    name = "DatabaseAction"

    def __init__(self, database):
        super().__init__()
        self.database = database

    def perform(self, paste, analyzer_name=None):
        self.database.store(paste)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class MailAnalyzer(RegexAnalyzer):
    """Analyzer to match on email addresses via regex"""
    name = "MailAnalyzer"

    def __init__(self, actions):
        # Regex taken from http://emailregex.com/
        regex = r"([a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-

import logging
from queue import Queue
from threading import Lock, Event

from pastepwn.util import start_thread


class ScrapingHandler(object):
    """Class to handle all the given scrapers to fetch pastes from different sources"""

    def __init__(self, paste_queue=None, exception_event=None):
        self.logger = logging.getLogger(__name__)
        self.running = False
        self.__exception_event = exception_event or Event()
        self.paste_queue = paste_queue or Queue()
        self.__lock = Lock()
        self.__threads = []
        self.scrapers = []

    def add_scraper(self, scraper):
        self.scrapers.append(scraper)

    def start(self):
        """Starts scraping pastes from the provided sources"""
        with self.__lock:
            if not self.running:
                # There needs to be at least one scraper
                if len(self.scrapers) == 0:
                    self.logger.warning("No scrapers added! At least one scraper must be added prior to use!")
                    return None

                self.running = True
                # Start all scraper threads
                for scraper in self.scrapers:
                    thread = start_thread(scraper.start, scraper.name, paste_queue=self.paste_queue, exception_event=self.__exception_event)
                    self.__threads.append(thread)

                # Return the update queue so the main thread can insert updates
                return self.paste_queue

    def stop(self):
        with self.__lock:
            if not self.running:
                return

            self.logger.debug("Stopping scrapers...")
            self.running = False
            for scraper in self.scrapers:
                scraper.stop()
            self._join_threads()

    def _join_threads(self):
        """End all threads and join them back into the main thread"""
        for thread in self.__threads:
            self.logger.debug("Joining thread {0}".format(thread.name))
            thread.join()
            self.logger.debug("Thread {0} has ended".format(thread.name))

        self.__threads = []



# -*- coding: utf-8 -*-
import logging
from threading import Lock

from requests import Session, utils
from requests.exceptions import Timeout


class Request(object):
    _instance = None
    _initialized = False
    _lock = Lock()

    def __new__(cls, *args, **kwargs):
        # override method to implement singleton
        # source: http://alacret.blogspot.com/2015/04/python-thread-safe-singleton-pattern.html
        if Request._instance is None:
            with Request._lock:
                if Request._instance is None:
                    Request._instance = super(Request, cls).__new__(cls)
        return Request._instance

    def __init__(self, proxies=None):
        if not self._initialized:
            self.logger = logging.getLogger(__name__)
            self.session = Session()
            self.proxies = proxies
            self.logger.info("Using the following custom proxies: {}".format(proxies))
            self.logger.info("Using the following system proxies: {}".format(utils.get_environ_proxies("https://example.com")))
            self._initialized = True

    def _request_wrapper(self, data, timeout, *args, **kwargs):
        headers = {
            "User-Agent": "pastepwn (https://github.com/d-Rickyy-b/pastepwn)"
        }

        try:
            response = self.session.request(headers=headers, proxies=self.proxies, data=data, timeout=timeout, *args, **kwargs)
            response_data = response.content.decode("utf-8")
        except Timeout:
            self.logger.warning("Timeout while requesting {0}!".format(kwargs.get("url")))
            return ""

        return response_data

    def get(self, url, data=None, timeout=5):
        return self._request_wrapper(method="GET", url=url, data=data, timeout=timeout)

    def post(self, url, data=None, timeout=5):
        return self._request_wrapper(method="POST", url=url, data=data, timeout=timeout)

    def put(self, url, data=None, timeout=5):
        return self._request_wrapper(method="PUT", url=url, data=data, timeout=timeout)

    def delete(self, url, data=None, timeout=5):
        return self._request_wrapper(method="DELETE", url=url, data=data, timeout=timeout)



# -*- coding: utf-8 -*-
import logging

import pymongo
from pymongo.errors import ConnectionFailure

from .abstractdb import AbstractDB


class MongoDB(AbstractDB):

    def __init__(self, ip="127.0.0.1", port=27017, dbname="pastepwn", collectionname="pastes"):
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self.logger.debug("Initializing MongoDB - {0}:{1}".format(ip, port))
        self.db = pymongo.MongoClient(ip, port, serverSelectionTimeoutMS=5000)

        try:
            self.db.admin.command("ismaster")
        except ConnectionFailure as e:
            self.logger.error(e)
            raise e

        self.logger.debug("Connected to database!")

        self.db = self.db[dbname]
        self.collection = self.db[collectionname]
        self.collection.create_index([('key', pymongo.ASCENDING)], unique=True)

    def _insert_data(self, data):
        self.collection.insert_one(data)

    def _get_data(self, key, value):
        return self.collection.find({key: value})

    # TODO def update_data(self, )

    # TODO def delete_data(self,)

    def count(self, key, value):
        return self.collection.find({key: value}).count()

    def count_all(self):
        return self.collection.count()

    def store(self, paste):
        self.logger.debug("Storing paste {0}".format(paste.key))

        try:
            self._insert_data(paste.to_dict())
        except pymongo.errors.DuplicateKeyError:
            self.logger.debug("Duplicate key '{0}' - Not storing paste".format(paste.key))

    def get(self, key):
        return self._get_data("key", key)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class SHAHashAnalyzer(RegexAnalyzer):
    """Analyzer to match SHA password hashes via regex"""
    name = "SHAHashAnalyzer"

    def __init__(self, actions):
        regex = r"[a-f0-9]{40}"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
from .basicanalyzer import BasicAnalyzer


class WordAnalyzer(BasicAnalyzer):
    """Analyzer to match the content of a paste via regular expressions"""
    name = "WordAnalyzer"

    def __init__(self, actions, word, blacklist=None, case_sensitive=False):
        super().__init__(actions, "{0} ({1})".format(self.name, word))
        self.word = word
        self.blacklist = blacklist or []
        self.case_sensitive = case_sensitive

    def _blacklist_word_found(self, text):
        if self.case_sensitive:
            text = text.lower()
            self.blacklist = [x.lower() for x in self.blacklist]

        for word in self.blacklist:
            if word in text:
                return True

        return False

    def match(self, paste):
        """Check if the specified word is part of the paste text"""
        paste_content = paste.body

        if self._blacklist_word_found(paste_content):
            return False

        if self.case_sensitive:
            return self.word in paste_content
        else:
            return self.word.lower() in paste_content.lower()



# -*- coding: utf-8 -*-
from .basicaction import BasicAction


class GenericAction(BasicAction):
    """Action to execute a custom function"""
    name = "GenericAction"

    def __init__(self, func):
        super().__init__()
        self.func = func

    def perform(self, paste, analyzer_name=None):
        self.func(paste)



# -*- coding: utf-8 -*-
import os

from .basicaction import BasicAction


class SaveFileAction(BasicAction):
    """Action to save a file to the disk"""
    name = "SaveFileAction"

    def __init__(self, path):
        super().__init__()
        self.path = path

    def perform(self, paste, analyzer_name=None):
        if not os.path.exists(self.path):
            os.makedirs(self.path)

        with open(os.path.join(self.path), "w") as file:
            file.write(str(paste))



# -*- coding: utf-8 -*-
from .basicanalyzer import BasicAnalyzer


class AlwaysTrueAnalyzer(BasicAnalyzer):
    """Analyzer which always matches a paste to perform actions on every paste"""
    name = "AlwaysTrueAnalyzer"

    def __init__(self, actions):
        """
        Analyzer which always matches a paste to perform actions on every paste
        :param actions: A single action or a list of actions to be executed on every paste
        """
        super().__init__(actions)

    def match(self, paste):
        """Always returns True to match every paste available"""
        return True



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.ibananalyzer import IBANAnalyzer


class TestIBANAnalyzer(unittest.TestCase):
    def setUp(self):
        self.cut = IBANAnalyzer(None, True)
        self.obj = mock.Mock()

    def test_validation_positive(self):
        self.assertTrue(self.cut._validate_iban("DE89 3704 0044 0532 0130 00"))
        self.assertTrue(self.cut._validate_iban("AT61 1904 3002 3457 3201"))
        self.assertTrue(self.cut._validate_iban("GB82-WEST-1234-5698-7654-32"))
        self.assertTrue(self.cut._validate_iban("NL20INGB0001234567"))

    def test_validation_negative(self):
        self.assertFalse(self.cut._validate_iban("FR14 2004 1010 0505 0001 3"))
        self.assertFalse(self.cut._validate_iban("XX00 1234 5678 9012 3456 7890 1234 5678 90"))
        self.assertFalse(self.cut._validate_iban("YY00123456789012345678901234567890"))
        self.assertFalse(self.cut._validate_iban("XX22YYY1234567890123"))
        self.assertFalse(self.cut._validate_iban("foo@i.ban"))

    def test_match_middle(self):
        self.obj.body = "This is inside DE89 3704 0044 0532 0130 00 a long paste."
        self.assertTrue(self.cut.match(self.obj))

    def test_match_end(self):
        self.obj.body = "This is at the beginning AT61 1904 3002 3457 3201"
        self.assertTrue(self.cut.match(self.obj))

    def test_match_beginning(self):
        self.obj.body = "NL20INGB0001234567 and this at the end."
        self.assertTrue(self.cut.match(self.obj))

    def test_match_single(self):
        self.obj.body = "GB82-WEST-1234-5698-7654-32"
        self.assertTrue(self.cut.match(self.obj))

    def test_match_multiple(self):
        self.obj.body = "This has DE89 3704 0044 0532 0130 00 two IBAN NL20INGB0001234567 in it."
        self.assertTrue(self.cut.match(self.obj))

    def test_match_validation(self):
        self.cut = IBANAnalyzer(None, False)
        self.obj.body = "This is inside FR14 2004 1010 0505 0001 3 a paste."
        self.assertTrue(self.cut.match(self.obj))

        self.cut = IBANAnalyzer(None, True)
        self.obj.body = "This is inside FR14 2004 1010 0505 0001 3 a paste."
        self.assertFalse(self.cut.match(self.obj))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-


class BasicAnalyzer(object):
    """Basic analyzer class"""
    name = "BasicAnalyzer"

    def __init__(self, actions, identifier=None):
        """
        Basic analyzer which is extended to create other analyzer subclasses
        :param actions: A single action or a list of actions to be executed on every paste
        :param identifier: The name or unique identifier for this specific analyzer
        """
        if actions is None:
            self.actions = []
        elif isinstance(actions, list):
            self.actions = actions
        else:
            self.actions = [actions]
        self.identifier = identifier or self.name

    def add_action(self, action):
        """
        Adds a new action to the already present actions
        :param action: New action to add to the present actions
        :return: None
        """
        self.actions.append(action)

    def match(self, paste):
        """
        Checks if a certain paste is matched by the conditions set for this analyzer
        :param paste: A :class:`pastepwn.core.paste` object which should be matched
        :return: :obj:`bool` if the paste has been matched
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-
import re
import urllib.error
import urllib.request

from .basicanalyzer import BasicAnalyzer


class URLAnalyzer(BasicAnalyzer):
    """
    This analyzer detects url-like patterns using regex,
    and then optionally attempts to verify that they can
    be resolved.
    """
    name = "URLAnalyzer"

    def __init__(self, actions, regex, resolve=False):
        super().__init__(actions, regex)
        self.regex = re.compile(regex)
        self.resolve = resolve  # Should we try to resolve the URLs?

    def _resolve_url(self, url):
        """
        A helper method that tries to resolve a given URL.
        Returns False if the URL cannot be resolved.
        """

        # If the url doesn't start with a protocol, we'll test with http and https.
        if not url.startswith("http"):
            for protocol in ("http", "https"):
                # If adding the protocol makes this resolve, the url works.
                if self._resolve_url("{0}://{1}".format(protocol, url)):
                    return True
            else:
                return False

        # Otherwise, let's just try resolving it.
        else:
            try:
                urllib.request.urlopen(url)
            except urllib.error.URLError:
                return False
            return True

    def match(self, paste):
        """ Check if the URL matches a certain regex, and then optionally try to resolve it. """
        paste_content = paste.body
        match = self.regex.search(paste_content)
        is_regex = match is not None

        if match:
            url = match.group(0)

        # Optionally check if the url resolves.
        if is_regex and self.resolve:
            return self._resolve_url(url)

        return is_regex



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

packages = find_packages(exclude=['tests*'])


# Taken from https://github.com/python-telegram-bot/python-telegram-bot/blob/9d99660ba95b103b3e1dc80414a5ce2fd805260b/setup.py#L9
def requirements():
    """Build the requirements list for this project"""
    requirements_list = []

    with open('requirements.txt') as reqs:
        for install in reqs:
            requirements_list.append(install.strip())

        return requirements_list


with open("README.md", "r", encoding="utf-8") as file:
    readme = file.read()

# Taken from https://packaging.python.org/guides/single-sourcing-package-version/
version = {}
version_file = os.path.join('pastepwn', 'version.py')
with open(version_file, "r", encoding="utf-8") as file:
    exec(file.read(), version)

    setup(name='pastepwn',
          version=version['__version__'],
          install_requires=requirements(),
          keywords='python pastebin scraping osint framework',
          description='Python framework to scrape PasteBin pastes and analyze them',
          long_description=readme,
          long_description_content_type='text/markdown',
          url='https://github.com/d-Rickyy-b/pastepwn',
          author='d-Rickyy-b',
          author_email='pastepwn@rickyy.de',
          license='MIT',
          packages=packages,
          include_package_data=True,
          classifiers=[
              'Development Status :: 5 - Production/Stable',
              'Environment :: Console',
              'Intended Audience :: Developers',
              'Intended Audience :: Science/Research',
              'License :: OSI Approved :: MIT License',
              'Operating System :: OS Independent',
              'Topic :: Software Development :: Libraries :: Python Modules',
              'Topic :: Security',
              'Topic :: Internet',
              'Programming Language :: Python',
              'Programming Language :: Python :: 3.5',
              'Programming Language :: Python :: 3.6',
              'Programming Language :: Python :: 3.7'
          ], )



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class BcryptHashAnalyzer(RegexAnalyzer):
    """Analyzer to match bcrypt password hashes via regex"""
    name = "BcryptHashAnalyzer"

    def __init__(self, actions):
        regex = r"\$2[ayb]\$.{56}"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
from threading import Event
from time import sleep


class BasicScraper(object):
    name = "BasicScraper"

    def __init__(self, exception_event=None):
        self.running = False
        self._stop_event = Event()
        self._exception_event = exception_event or Event()

    def init_exception_event(self, exception_event):
        self._exception_event = exception_event

    def start(self, paste_queue):
        raise NotImplementedError

    def stop(self):
        if self.running:
            self._stop_event.set()
            while self.running:
                sleep(0.1)
            self._stop_event.clear()



# -*- coding: utf-8 -*-

import unittest

from pastepwn import Paste


class PasteTest(unittest.TestCase):

    def setUp(self):
        p = {"scrape_url": "https://scrape.pastebin.com/api_scrape_item.php?i=0CeaNm8Y",
             "full_url": "https://pastebin.com/0CeaNm8Y",
             "date": "1442911802",
             "key": "0CeaNm8Y",
             "size": "890",
             "expire": "1442998159",
             "title": "Once we all know when we goto function",
             "syntax": "java",
             "user": "admin",
             "body": "This is a test for pastepwn"}

        self.p = p
        self.paste = Paste(p.get("key"),
                           p.get("title"),
                           p.get("user"),
                           p.get("size"),
                           p.get("date"),
                           p.get("expire"),
                           p.get("syntax"),
                           p.get("scrape_url"),
                           p.get("full_url"))

    def tearDown(self):
        pass

    def test_init_paste(self):
        self.assertEqual(self.p.get("key"), self.paste.key)
        self.assertEqual(self.p.get("title"), self.paste.title)
        self.assertEqual(self.p.get("user"), self.paste.user)
        self.assertEqual(self.p.get("size"), self.paste.size)
        self.assertEqual(self.p.get("date"), self.paste.date)
        self.assertEqual(self.p.get("expire"), self.paste.expire)
        self.assertEqual(self.p.get("syntax"), self.paste.syntax)
        self.assertEqual(self.p.get("scrape_url"), self.paste.scrape_url)
        self.assertEqual(self.p.get("full_url"), self.paste.full_url)
        self.assertEqual(None, self.paste.body)

    def test_set_body(self):
        my_body = "This is a test for pastepwn"
        self.paste.set_body(my_body)
        self.assertEqual(my_body, self.paste.body)

    def test_to_dict(self):
        my_body = "This is a test for pastepwn"
        self.paste.set_body(my_body)

        paste_dict = self.paste.to_dict()
        self.assertEqual(self.p, paste_dict)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class CreditCardAnalyzer(RegexAnalyzer):
    """Analyzer to match Credit Cards"""

    def __init__(self, action):
        """
        Analyzer to match Credit Cards
        :param action: Single action or list of actions to be executed when a paste matches
        """
        # Regex taken from https://www.regular-expressions.info/creditcard.html
        regex = r"^4[0-9]{12}(?:[0-9]{3})?$|^3[47][0-9]{13}$|" \
                r"^(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}$|" \
                r"^3(?:0[0-5]|[68][0-9])[0-9]{11}$|" \
                r"^6(?:011|5[0-9]{2})[0-9]{12}|^(?:2131|1800|35\d{3})\d{11}$"

        super().__init__(action, regex)



# -*- coding: utf-8 -*-
import logging

import mysql.connector

from .abstractdb import AbstractDB


class MysqlDB(AbstractDB):

    def __init__(self, ip="127.0.0.1", port=3306, unix_socket=None, dbname="pastepwn", username=None, password=None, timeout=10):
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self.logger.debug("Initializing MySQLDB - {0}:{1}".format(ip, port))

        # https://dev.mysql.com/doc/connector-python/en/connector-python-connectargs.html
        if unix_socket:
            self.db = mysql.connector.connect(
                host=ip,
                user=username,
                passwd=password,
                unix_socket=unix_socket,
                connection_timeout=timeout
            )
        else:
            self.db = mysql.connector.connect(
                host=ip,
                port=port,
                user=username,
                passwd=password,
                database=dbname,
                connection_timeout=timeout
            )

        self.cursor = self.db.cursor()
        # self._create_db(dbname) # Not used because of possible SQLI
        self._create_tables()

        self.logger.debug("Connected to database!")

    def _create_db(self, dbname):
        # Currently I found no other way to insert the database name into the sql statement
        # With the following code a simple SQL Injection would be possible - question is, why would a user do this to his own database?
        # Nevertheless I don't want to put this into production that way. I'll keep the code but remove the call to it.
        self.logger.info("Creating database '{0}' (if not exists)".format(self.dbname))
        self.cursor.execute("""CREATE DATABASE IF NOT EXISTS %s;""" % self.dbname)
        self.cursor.execute("""USE %s;""" % self.dbname)
        self.db.commit()

    def _create_tables(self):
        # Although the length of 'key' should never exceed 8 chars,
        # making it longer prevents from future issues.
        self.cursor.execute("""CREATE TABLE IF NOT EXISTS `pastes` (
                                    `key` VARCHAR(30) NOT NULL UNIQUE,
                                    `title` TEXT,
                                    `user` TEXT,
                                    `size` INTEGER,
                                    `date` INTEGER,
                                    `expire` INTEGER,
                                    `syntax` TEXT,
                                    `scrape_url` TEXT,
                                    `full_url` TEXT,
                                    `body` TEXT,
                                    PRIMARY KEY(`key`));""")
        self.db.commit()

    def _insert_data(self, paste):
        self.cursor.execute("INSERT INTO `pastes` (`key`, `title`, `user`, `size`, `date`, `expire`, `syntax`, `scrape_url`, `full_url`, `body`) "
                            "VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s);",
                            (paste.key,
                             paste.title,
                             paste.user,
                             paste.size,
                             paste.date,
                             paste.expire,
                             paste.syntax,
                             paste.scrape_url,
                             paste.full_url,
                             paste.body))
        self.db.commit()

    def _get_data(self, key, value):
        raise NotImplementedError

    def count(self, key, value):
        # TODO add filter to counting
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def count_all(self):
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def store(self, paste):
        self.logger.debug("Storing paste {0}".format(paste.key))

        try:
            self._insert_data(paste)
        except Exception as e:
            self.logger.debug("Exception '{0}'".format(e))

    def get(self, key):
        return self._get_data("key", key)



# -*- coding: utf-8 -*-
import logging.handlers

from .basicaction import BasicAction


class SyslogAction(BasicAction):
    """Action to log a paste to the syslog"""
    name = "SyslogAction"

    def __init__(self, syslog_address="/dev/log"):
        """
        This sets up a syslogger, which defaults to /dev/log.
        That means that it will work on most linux systems,
        but will not work on for example OSX.

        For OSX, the correct syslog address would be
        /var/run/syslog
        """
        super().__init__()
        self.logger = logging.getLogger('SyslogLogger')
        self.logger.setLevel(logging.DEBUG)

        handler = logging.handlers.SysLogHandler(address=syslog_address)
        self.logger.addHandler(handler)

    def perform(self, paste, analyzer_name=None):
        self.logger.debug("New Paste matched: {0}".format(paste))



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.alwaystrueanalyzer import AlwaysTrueAnalyzer


class TestAlwaysTrueAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = AlwaysTrueAnalyzer(None)
        self.obj = mock.Mock()

    def test_match(self):
        self.obj.body = "Test"
        self.assertTrue(self.analyzer.match(self.obj))

        self.obj.body = None
        self.assertTrue(self.analyzer.match(self.obj))

        self.obj.body = ""
        self.assertTrue(self.analyzer.match(self.obj))

        self.obj = None
        self.assertTrue(self.analyzer.match(self.obj))

    def test_actions_present(self):
        analyzer = AlwaysTrueAnalyzer(self.obj)
        self.assertEqual([self.obj], analyzer.actions)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.basicanalyzer import BasicAnalyzer


class TestBasicAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = BasicAnalyzer(None)
        self.mock_action = mock.Mock()

    def test_add_action(self):
        """Check if it's possible to add actions to an analyzer"""
        mock_action2 = mock.Mock
        analyzer = BasicAnalyzer(self.mock_action)
        self.assertEqual([self.mock_action], analyzer.actions)

        analyzer.add_action(mock_action2)
        self.assertTrue(isinstance(analyzer.actions, list))
        self.assertEqual([self.mock_action, mock_action2], analyzer.actions)

    def test_match(self):
        """Check if an exception is raised when trying to call the match function"""
        self.mock_action.body = "Test"
        self.assertRaises(NotImplementedError, self.analyzer.match, self.mock_action)

    def test_initialization(self):
        """Check if the initialization of the BasicAnalyzer works as intended"""
        actions = mock.Mock()
        analyzer = BasicAnalyzer(actions)
        self.assertEqual([actions], analyzer.actions)

    def test_empty_initialization(self):
        analyzer = BasicAnalyzer(None)
        self.assertEqual([], analyzer.actions)

    def test_single_initialization(self):
        analyzer = BasicAnalyzer(self.mock_action)
        self.assertEqual([self.mock_action], analyzer.actions)

    def test_multi_initialization(self):
        mock_action2 = mock.Mock()
        actions = [self.mock_action, mock_action2]
        analyzer = BasicAnalyzer(actions)
        self.assertEqual([self.mock_action, mock_action2], analyzer.actions)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import json


class APIResponse(object):

    def __init__(self, status, message):
        self.status = status
        self.message = message
        self.error = None

    def __str__(self):
        return json.dumps({"status": self.status, "error": self.error, "message": self.message})



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.mailanalyzer import MailAnalyzer


class TestMailAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = MailAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        self.paste.body = "test@example.com"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "This is a test with a mail address in the end test@example.com"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "asdf@testmail.com This is a test with a mail address in the beginning"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "This is a test with a mail@address.com in the middle"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "This is a test without a mail@address anywhere"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
import re
from string import Template

from pastepwn.util import Request, DictWrapper
from .basicaction import BasicAction


class TelegramAction(BasicAction):
    """Action to send a Telegram message to a certain user or group"""
    name = "TelegramAction"

    def __init__(self, token, receiver, custom_payload=None, template=None):
        super().__init__()
        self.logger = logging.getLogger(__name__)

        if not re.match(r"[0-9]+:[a-zA-Z0-9\-_]+", token) or token is None:
            raise ValueError("Bot token not correct or None!")

        self.token = token
        self.receiver = receiver
        self.custom_payload = custom_payload
        if template is not None:
            self.template = Template(template)
        else:
            self.template = None

    def perform(self, paste, analyzer_name=None):
        """Send a message via a Telegram bot to a specified user, without checking for errors"""
        r = Request()
        if self.template is None:
            text = "New paste matched by analyzer '{0}' - Link: {1}".format(analyzer_name, paste.full_url)
        else:
            paste_dict = paste.to_dict()
            paste_dict["analyzer_name"] = analyzer_name
            text = self.template.safe_substitute(DictWrapper(paste_dict))

        api_url = "https://api.telegram.org/bot{0}/sendMessage?chat_id={1}&text={2}".format(self.token, self.receiver, text)
        r.get(api_url)
